@echo off
REM *************************************
REM *** Change these input-parameters ***
REM *************************************
set opening_plies=16
set games=ccc22-rapid-entry-league.pgn
set reference_eng="Arasan"
set /A reference_elo=0

REM **********************************************
REM *** Start of the batch-tool, do not change ***
REM **********************************************

REM *** Cleanup and change the working path to the bin-folder ***
del Gamepairs_final.pgn 2>nul
del Gamepairs_head-to-head.txt 2>nul
del Gamepairs_rating.txt 2>nul
cd ./bin
del box*.pgn 2>nul
del Gamepairs_final.pgn 2>nul
del cut.pgn 2>nul
del winwhite.pgn 2>nul
del winblack.pgn 2>nul
del losswhite.pgn 2>nul
del lossblack.pgn 2>nul
del draw.pgn 2>nul
del unique_draw.pgn 2>nul
del won_pairs.pgn 2>nul
del loss_pairs.pgn 2>nul
del draw_pairs.pgn 2>nul
del work01.pgn 2>nul
del work02.pgn 2>nul
del 20_00_eng1.pgn 2>nul
del 15_05_eng1.pgn 2>nul
del 15_05_eng1_A.pgn 2>nul
del 15_05_eng1_B.pgn 2>nul
del 10_10_eng1_A.pgn 2>nul
del 10_10_eng1_B.pgn 2>nul
del 10_10_eng1_C.pgn 2>nul
del 10_10_eng1_D.pgn 2>nul
del 10_10_eng1_E.pgn 2>nul
del 10_10_eng1_F.pgn 2>nul
del 05_15_eng1.pgn 2>nul
del 05_15_eng1_A.pgn 2>nul
del 05_15_eng1_B.pgn 2>nul
del 00_20_eng1.pgn 2>nul
del final_pairs.pgn 2>nul


@echo ************************************************************
@echo ***     Gamepair Rescorer V1.5 by Stefan Pohl (SPCC)     ***
@echo ***              (C) 2023, www.sp-cc.de                  ***
@echo ************************************************************
@echo ************************************************************
@echo ************************************************************

REM *** Cut all games after the opening-moves, and delete comments and useless tags ***
@echo Step 1: Cut all games after the opening-moves and clean pgn
pgn-extract --quiet -C -N -V --seven --plylimit %opening_plies% ../%games% --output cut.pgn

REM *** Split the new shortened gamefile into head-to-head files ***
@echo ************************************************************
@echo Step 2: Split gamefile into head-to-head files
pairSplit cut.pgn >nul

REM *** Count, how many files pairSplit made *** 
set /A "filenumber=0"
for /F "delims=" %%A IN (manifest-box) do set /A "filenumber+=1"
set /A show_fn=%filenumber%
set /A filenumber=%filenumber%-1
@echo Number of head-to-head files generated: %show_fn%
@echo ************************************************************

REM *** Check, if the number of head-to-files is not too big ***
if %show_fn% GEQ 996 (
@echo ERROR: Overflow! Too much head-to-head files
@echo Program terminated...
@echo ************************************************************
set /A overflow=1
goto exitloop01
) else (
set /A overflow=0
)

REM *** Start working on each head-to-head file, sequentially ***
@echo Step 3: Start processing head-to head files sequentially
set /A filecounter=-1
set /A all_drawpairs=0
set /A all_winwinpairs=0
:loop01
set /A filecounter=%filecounter%+1
set /A show_fc=%filecounter%+1

@echo Processing head-to-head file %show_fc% of %show_fn%

REM *** Build the filename (boxXXX.pgn) for the next file ***
if %filecounter% GEQ 100 set filename=box%filecounter%.pgn
if %filecounter% LEQ 99 set filename=box0%filecounter%.pgn
if %filecounter% LEQ 9 set filename=box00%filecounter%.pgn

REM *** Extract one of the two engine-names for further processing ***
nameList %filename% >nul
set /p engine1=<outName

REM **************************************************************
REM *** Do the gamepairing operations on the head-to-head-file ***
REM **************************************************************

REM *** Find won, lost and draw games (white and black) (from the point of view of one engine) ***
pgn-extract --quiet -Tw"%engine1%" -Tr1-0 %filename% --output winwhite.pgn
pgn-extract --quiet -Tb"%engine1%" -Tr0-1 %filename% --output winblack.pgn
pgn-extract --quiet -Tb"%engine1%" -Tr1-0 %filename% --output lossblack.pgn
pgn-extract --quiet -Tw"%engine1%" -Tr0-1 %filename% --output losswhite.pgn
pgn-extract --quiet -Tr1/2-1/2 %filename% --output draw.pgn

pgn-extract --quiet --fuzzydepth 0 -D draw.pgn --output unique_draw.pgn

REM *** Build the 5 possible results-gamebases (2-0, 1.5-0.5, 1.0-1.0, 0.5-1.5, 0-2) ***
pgn-extract --quiet -U -d20_00_eng1.pgn winwhite.pgn winblack.pgn 

set /A drawpairs=0
set /A winwinpairs=0
pgn-extract --quiet -U -d10_10_eng1_A.pgn draw.pgn 
CALL :countgames 10_10_eng1_A.pgn
set /A drawpairs=%c_numb_pgn%
pgn-extract --quiet -U -d10_10_eng1_B.pgn winwhite.pgn lossblack.pgn
pgn-extract --quiet -U -d10_10_eng1_C.pgn losswhite.pgn winblack.pgn
pgn-extract --quiet -o10_10_eng1_D.pgn 10_10_eng1_A.pgn 10_10_eng1_B.pgn
pgn-extract --quiet -o10_10_eng1_E.pgn 10_10_eng1_C.pgn 10_10_eng1_D.pgn
CALL :countgames 10_10_eng1_B.pgn
set /A winwinpairs=%c_numb_pgn%
CALL :countgames 10_10_eng1_C.pgn
set /A winwinpairs+=%c_numb_pgn%

set /A all_drawpairs+=%drawpairs%
set /A all_winwinpairs+=%winwinpairs%

textReplace 10_10_eng1_E.pgn 10_10_eng1_F.pgn "1-0" "1/2-1/2"
textReplace 10_10_eng1_F.pgn draw_pairs.pgn "0-1" "1/2-1/2"

pgn-extract --quiet -U -d15_05_eng1_A.pgn unique_draw.pgn winwhite.pgn 
pgn-extract --quiet -U -d15_05_eng1_B.pgn unique_draw.pgn winblack.pgn 
pgn-extract --quiet -o15_05_eng1.pgn 15_05_eng1_A.pgn 15_05_eng1_B.pgn

pgn-extract --quiet -U -d05_15_eng1_A.pgn unique_draw.pgn losswhite.pgn
pgn-extract --quiet -U -d05_15_eng1_B.pgn unique_draw.pgn lossblack.pgn
pgn-extract --quiet -o05_15_eng1.pgn 05_15_eng1_A.pgn 05_15_eng1_B.pgn

pgn-extract --quiet -U -d00_20_eng1.pgn losswhite.pgn lossblack.pgn 

REM *** Merge (2-0 and 1.5-0.5) and (0-2 and 0.5-1.5) gamebases into one file***
pgn-extract --quiet -owon_pairs.pgn 20_00_eng1.pgn 15_05_eng1.pgn
pgn-extract --quiet -oloss_pairs.pgn 00_20_eng1.pgn 05_15_eng1.pgn

REM *** Merge all 3 remaining gamepairs (W-D-L) into one file ***
pgn-extract --quiet -owork01.pgn won_pairs.pgn draw_pairs.pgn
pgn-extract --quiet -owork02.pgn work01.pgn loss_pairs.pgn 
pgn-extract --quiet -C -N -V work02.pgn --output final_pairs.pgn

REM **************************************************************

REM *** collect the gamepairs-data in one pgn-file (Gamepairs_final.pgn) ***
pgn-extract --quiet -aGamepairs_final.pgn final_pairs.pgn

REM *** delete the processed head-to-head file, which is no longer needed ***
del %filename%

REM *** All head-to-head files done? Then exit, otherwise jump back and proceed ***
if %filecounter% GEQ %filenumber% goto exitloop01
goto loop01
:exitloop01

REM *** Cleanup ***
del cut.pgn 2>nul
del box*.pgn 2>nul
del winwhite.pgn 2>nul
del winblack.pgn 2>nul
del losswhite.pgn 2>nul
del lossblack.pgn 2>nul
del draw.pgn 2>nul
del unique_draw.pgn 2>nul
del won_pairs.pgn 2>nul
del loss_pairs.pgn 2>nul
del draw_pairs.pgn 2>nul
del work01.pgn 2>nul
del work02.pgn 2>nul
del 20_00_eng1.pgn 2>nul
del 15_05_eng1.pgn 2>nul
del 15_05_eng1_A.pgn 2>nul
del 15_05_eng1_B.pgn 2>nul
del 10_10_eng1_A.pgn 2>nul
del 10_10_eng1_B.pgn 2>nul
del 10_10_eng1_C.pgn 2>nul
del 10_10_eng1_D.pgn 2>nul
del 10_10_eng1_E.pgn 2>nul
del 10_10_eng1_F.pgn 2>nul
del 05_15_eng1.pgn 2>nul
del 05_15_eng1_A.pgn 2>nul
del 05_15_eng1_B.pgn 2>nul
del 00_20_eng1.pgn 2>nul
del final_pairs.pgn 2>nul
del excludeBox.pgn 2>nul
del manifest-box 2>nul
del outName 2>nul
del outName2 2>nul
del outName3 2>nul

REM *** If an overflow occured, it makes no sense to start ORDO ***
if %overflow% GEQ 1 goto exit

@echo ************************************************************
@echo Step 4: Elo-calculating with ORDO

ordo-win64 -n 1 -J -j "../Gamepairs_head-to-head.txt" -q -N 0,1 -U "0,1,2,4,7,8,9,5,6" -a %reference_elo% -A %reference_eng% -o "../Gamepairs_rating.txt" -p Gamepairs_final.pgn -s500 >nul
move Gamepairs_final.pgn "../Gamepairs_final.pgn" >nul
cd ../

CALL :countgames Gamepairs_final.pgn
set /A all_gamepairs=%c_numb_pgn%
set /A all_drawn_gamepairs=%all_drawpairs%+%all_winwinpairs%

@echo ------------------------------------------------------------------- >>Gamepairs_rating.txt
@echo --- Number of all Gamepairs          : %all_gamepairs% >>Gamepairs_rating.txt
CALL :percent %all_gamepairs% %all_drawn_gamepairs%
@echo --- Number of drawn Gamepairs overall: %all_drawn_gamepairs% (= %percent%) >>Gamepairs_rating.txt
CALL :percent %all_gamepairs% %all_winwinpairs%
@echo --- Number of 1:1 drawn Gamepairs    : %all_winwinpairs% (= %percent%) >>Gamepairs_rating.txt
CALL :percent %all_gamepairs% %all_drawpairs%
@echo --- Number of 2-draws drawn Gamepairs: %all_drawpairs% (= %percent%) >>Gamepairs_rating.txt
@echo ------------------------------------------------------------------- >>Gamepairs_rating.txt

@echo ************************************************************
type Gamepairs_rating.txt
@echo ************************************************************
:exit
@pause
goto :EOF

:countgames
set /A c_numb_pgn=0
for /f "tokens=2,* delims= " %%G in ('find /C "[White " %~1') do set /A c_numb_pgn=%%H
exit /B 0

:percent
if %~1 LEQ 0 (
set percent=00.00%%
set /A percentx100=0
EXIT /B 0
)
set /A l_base=((1000000000/%~1)*%~2)
set /A l_percent=%l_base%/1000000
set /A l_rest1=%l_percent%%%10
set /A l_percent=%l_percent%/10
set /A l_secdig_pc=%l_base%/100000
set /A l_rest2=%l_secdig_pc%%%10
set /A l_thirddig_pc=%l_base%/10000
set /A l_rest3=%l_thirddig_pc%%%10

REM *** Round 2nd digit up, if 3rd digit is GEQ 5 and round 1st digit and percent-value up, if necessary
if %l_rest3% GEQ 5 set /A l_rest2+=1
if %l_rest2% GEQ 10 (
  set /A l_rest2-=10
  set /A l_rest1+=1
)
if %l_rest1% GEQ 10 (
  set /A l_rest1-=10
  set /A l_percent+=1
)

if %l_percent% LSS 10 (
  set percent=0%l_percent%.%l_rest1%%l_rest2%%%
) else (
  set percent=%l_percent%.%l_rest1%%l_rest2%%%
)
set /A percentx100=(%l_percent%*100)+(%l_rest1%*10)+%l_rest2%

if %l_percent% GTR 99 (
set percent=100.0%%
set /A percentx100=10000
)
EXIT /B 0